#
# Copyright (C) 2021 Associated Universities, Inc. Washington DC, USA.
#
# This file is part of NRAO Workspaces.
#
# Workspaces is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Workspaces is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Workspaces.  If not, see <https://www.gnu.org/licenses/>.
"""
Tests for Vela Forger
"""
from unittest.mock import mock_open, patch

import pytest
from vela.forger import VelaLog, VelaProduct, read_metadata


class TestVelaLog:
    def test_forge_logs(self):
        parameters_cal = read_metadata("test/test.json")
        with patch("vela.forger.VelaLog.cal_logs") as cal_logs:
            VelaLog("test.json", "standard_cal").forge_logs(parameters=parameters_cal)
            assert cal_logs.call_count == 1

        parameters_img = read_metadata("test/image-metadata.json")
        with patch("vela.forger.VelaLog.img_logs") as img_logs:
            VelaLog("image-metadata.json", "standard_img").forge_logs(parameters=parameters_img)
            assert img_logs.call_count == 1

    @patch("os.chdir")
    @patch("arrow.now")
    def test_cal_logs(self, mock_arrow, mock_os):
        with patch("builtins.open", mock_open()) as o:
            VelaLog("test.json", "standard_cal").cal_logs()
            assert o.call_count == 3
            handle = o()
            assert handle.write.call_count == 3

    @patch("os.chdir")
    @patch("arrow.now")
    def test_img_logs(self, mock_arrow, mock_os):
        with patch("builtins.open", mock_open()) as o:
            VelaLog("image-metadata.json", "standard_img").img_logs()
            assert o.call_count == 3
            handle = o()
            assert handle.write.call_count == 3


class TestVelaProduct:
    @pytest.mark.skip("Now ignores mock....")
    def test_forge_products(self):
        parameters_cal = read_metadata("test/test.json")
        with patch("vela.forger.VelaProduct.cal_products") as cal_products:
            VelaProduct("test.json", "standard_cal").forge_products(parameters=parameters_cal)
            assert cal_products.call_count == 1

        parameters_img = read_metadata("test/image-metadata.json")
        with patch("vela.forger.VelaProduct.img_products") as img_products:
            VelaProduct("image-metadata.json", "standard_img").forge_products(parameters=parameters_img)
            assert img_products.call_count == 1

    @patch("os.chdir")
    @patch("vela.forger.VelaProduct.forge_weblog")
    @patch("vela.forger.VelaProduct.forge_measurement_set")
    def test_cal_products(self, mock_ms, mock_forge, mock_os):
        parameters = read_metadata("test/test.json")
        with patch("builtins.open", mock_open()) as o:
            VelaProduct("test.json", "standard_cal").cal_products(parameters=parameters)
            o.assert_called_once_with("brain_001.58099.678886747686.ms.calapply.txt", "x")
            handle = o()
            handle.write.assert_called_once_with("I am a calibration file.")

    @patch("os.mkdir")
    @patch("os.chdir")
    def test_forge_measurement_set(self, mock_chdir, mock_mkdir):
        with patch("builtins.open", mock_open()) as o:
            VelaProduct("test.json", "standard_cal").forge_measurement_set(sdm_id="brain_001.58099.678886747686")
            assert o.call_count == 10
            handle = o()
            assert handle.write.call_count == 10

            assert mock_mkdir.call_count == 6
            assert mock_chdir.call_count == 2

    @patch("os.mkdir")
    @patch("os.chdir")
    @patch("arrow.now")
    @patch("tarfile.open")
    def test_forge_weblog(self, mock_tar, mock_arrow, mock_os_chdir, mock_os_mkdir):
        with patch("builtins.open", mock_open()) as o:
            VelaProduct("test.json", "standard_cal").forge_weblog(path="/tmp/workspaces_tmp/testing")
            assert o.call_count == 2
            handle = o()
            assert handle.write.call_count == 2


def test_read_metadata():
    parameters = read_metadata("test/test.json")
    assert parameters["fileSetIds"] == "brain_001.58099.678886747686"
    assert parameters["workflowName"] == "std_calibration"
    assert parameters["systemId"] == "5"
    assert parameters["productLocator"] == "uid://evla/execblock/9cb0964b-ad6b-40ed-bd87-d08c502503e2"
    project = parameters.get("projectMetadata")
    assert project["projectCode"] == "Operations"
    assert project["title"] == ""
    assert project["startTime"] == "58099.6790081019"
    assert project["observer"] == "VLA Operations"
